// StegoWav - a tool for steganography on Microsoft wave files
//
// Copyright (C) 1996 by Peter Heist <heistp@rpi.edu>.  All rights reserved.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

import java.awt.*;
import java.io.*;
import java.lang.*;
import AudioCover;
import PasswordDialog;

public class GUIWindow extends Frame
    {
    final String VERSIONINFO = "StegoWav 1.7 (Peter Heist - 1996)";
    boolean inAnApplet = true;
    final String FILEOPENMENUITEM = "Open Cover...";
    final String FILEHIDEMENUITEM = "Hide File...";
    final String FILESAVEASMENUITEM = "Save Hidden File As...";
    final String FILECLOSEMENUITEM = "Close Cover...";
    final String FILEEXITMENUITEM = "Exit";
    TextArea textArea = new TextArea (VERSIONINFO, 3, 80);
    MenuItem fileOpenMenuItem = new MenuItem (FILEOPENMENUITEM);
    MenuItem fileHideMenuItem = new MenuItem (FILEHIDEMENUITEM);
    MenuItem fileSaveAsMenuItem = new MenuItem (FILESAVEASMENUITEM);
    MenuItem fileCloseMenuItem = new MenuItem (FILECLOSEMENUITEM);
    MenuItem fileExitMenuItem = new MenuItem (FILEEXITMENUITEM);
    String coverFile, coverDirectory;
    String secretFile, secretDirectory;
    String password;
    AudioCover ac;

    public GUIWindow()
	{
        Panel centerPanel = new Panel();
        setLayout(new BorderLayout());

	this.setResizable (false);

	// some default colors and fonts
	Color backColor = new Color (0xB0B0B0);
	Color foreColor = Color.black;
	this.setBackground (backColor);
	this.setForeground (foreColor);
	this.setFont (new Font("TimesRoman", Font.PLAIN, 12));

        //Set up the menu bar.
        MenuBar mb = new MenuBar ();
        Menu fm = new Menu ("File");
	fm.add (fileOpenMenuItem);
	fm.add (fileHideMenuItem);
	fileHideMenuItem.disable ();
	fm.add (fileSaveAsMenuItem);
	fileSaveAsMenuItem.disable ();
	fm.add (fileCloseMenuItem);
	fileCloseMenuItem.disable();
        fm.add (new MenuItem("-"));
	fm.add (fileExitMenuItem);
        mb.add (fm);

	// set the menu bar
        setMenuBar (mb);

        //Put a label and a text area in the right column.
        Panel p = new Panel();
	textArea.setEditable (false);
	textArea.setBackground (Color.white);
	
	// textArea.setBackground (backColor);
        p.setLayout(new BorderLayout());
        p.add("North", new Label("Messages:", Label.LEFT));
        p.add("Center", textArea);
	p.add("South", new Label("", Label.LEFT));
        centerPanel.add(p);
        add("Center", centerPanel);
	}

    public boolean action(Event event, Object arg)
	{
        if (event.target instanceof MenuItem)
	    {
	    // select a cover file
            if (((String)arg).equals(FILEOPENMENUITEM))
		{
		PasswordDialog pd = new PasswordDialog (this,
		    "Enter Passphrase for Encryption:");
		//pd.show();
		this.password = pd.getPassword();
	        FileDialog sfd = new FileDialog(this, "Select Cover Wave File...", 0);
	        sfd.show();

		if (null != sfd.getFile())
		    {
		    coverFile = sfd.getFile();
		    coverDirectory = sfd.getDirectory();
		    this.printMessage ("\nAnalyzing cover file...");
		    try 
		    	{
		    	ac = new AudioCover (password, coverDirectory + coverFile);
			}
		    catch (IOException e)
		    	{
		    	this.printMessage("\nfileOpen: " + e);
		    	}
		    catch (BadAudioFileException e)
		    	{
		    	this.printMessage("\nfileOpen: " + e);
		    	}

		    this.printMessage (coverFile + " has room for " +
			ac.bytesCanCover() + " bytes.");

		    if (ac.hasHiddenFile() == true)
		    	{
		    	fileSaveAsMenuItem.enable();
			this.printMessage("\nHidden file " + ac.getHiddenFileName() +
			    " found.");
			}
		    fileHideMenuItem.enable();
		    fileCloseMenuItem.enable();
		    }
		}

	    // hide secret message file
	    else if (((String)arg).equals(FILEHIDEMENUITEM))
	    	{
		FileDialog lfd = new FileDialog(this, "Select File smaller than roughly " +
		    ac.bytesCanCover() + " bytes to Hide...", 0);
		lfd.show();
		secretFile = lfd.getFile();
		secretDirectory = lfd.getDirectory();

		if (null != secretFile)
		    {
		    this.printMessage("\nHiding " + secretFile + " in " + coverFile +
			"...");
		    try 
		    	{
		    	ac.hideFile(secretDirectory, secretFile);
			}
		    catch (IOException e)
		    	{
		    	this.printMessage("fileHide: " + e);
		    	}
		    catch (BadAudioFileException e)
		    	{
		    	this.printMessage("fileHide: " + e);
		    	}

		    this.printMessage("done.");
		    }
		}

	    // recover secret message file
	    else if (((String)arg).equals(FILESAVEASMENUITEM))
	    	{
		if (true == ac.hasHiddenFile())
		    {
		    FileDialog sfd = new FileDialog(this, "Save File As...", 1);
		    sfd.setFile (ac.getHiddenFileName());
		    sfd.show ();
		    if (null != sfd.getFile())
		    	{
			this.printMessage("\nSaving " + ac.getHiddenFileName() + " as " +
			    sfd.getFile() + "...");
			try 
			    {
			    ac.saveHiddenFileAs(sfd.getDirectory() + sfd.getFile());
			    }
			catch (IOException e)
			    {
			    this.printMessage("fileSaveAs: " + e);
			    }
			this.printMessage("done.");
			}
		    }
		else
		    this.printMessage ("\nFailed to find secret file.");
	    	}

	    // close cover file
	    else if (((String)arg).equals(FILECLOSEMENUITEM))
	    	{
		try 
		    {
		    ac.close();
		    }
		catch (IOException e)
		    {
		    this.printMessage("\nfileClose: " + e);
		    }

	    	fileHideMenuItem.disable();
		fileSaveAsMenuItem.disable();
		fileCloseMenuItem.disable();
	    	}
	    
	    // exit
	    else if (((String)arg).equals(FILEEXITMENUITEM))
	    	{
		if (inAnApplet)
		    dispose();
		else
		    System.exit(0);
	    	}
	    }
        return true;
	}

    public boolean handleEvent(Event event)
	{
        //If we're running as an application, closing the window
        //should quit the application.
        if (event.id == Event.WINDOW_DESTROY)
	    {
            if (inAnApplet)
                dispose();
	    else
                System.exit(0);
	    }
        return super.handleEvent(event);
	}

    public void printMessage (String message)
    	{
    	textArea.appendText (message);
    	}
    }

