// StegoWav - a tool for steganography on Microsoft wave files
//
// Copyright (C) 1996 by Peter Heist <heistp@rpi.edu>.  All rights reserved.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

import java.crypt.*;
import java.io.*;
import WavFile;
import PRPG;

public class WavStat
    {
    static int histogramArraySize = 50;
    static int[] histogramArray = new int[histogramArraySize];

    public static void main(String argv[]) throws EOPException
    	{
	short[] sampleBuffer = new short[16384];
	byte lastLSB = (byte) 0x02;
	int runLength = 1;
	long runLengthEntries = 0;
	long runLengthTotal = 0;
	int bytesRead;
	//PRPG myPRPG = new PRPG(MessageDigest.hash("test", new MD5()), 10000);

	//do
	//    {
	//    } while (myPRPG.getNextIndex() != -1);
	//System.out.println("Finished generating 5000000 byte permutation.");

	for (int i = 0; i < histogramArraySize; i++)
	    {
	    histogramArray[i] = 0;
	    }

	try 
	    {
	    WavFile wf = new WavFile(argv[0], "r");
	    
	    // read the samples into an array of shorts and calculate run lengths

	    while ((bytesRead = wf.read(sampleBuffer)) != 0)
		{
		for (int i = 0; i < bytesRead; i++)
		    {
		    if (lastLSB == (byte) ((sampleBuffer[i]) & 0x01) )
			runLength++;
		    else
		    	{
			addToHistogram(runLength);
		    	runLengthEntries++;
			runLengthTotal += runLength;
			runLength = 1;
		    	}

		    lastLSB = (byte) ((sampleBuffer[i]) & 0x01);
		    }
		}
	    }
	catch (IOException e)
	    {
	    System.out.println("main: " + e);
	    }
	catch (BadAudioFileException e)
	    {
	    System.out.println("main: " + e);
	    }

	System.out.println("Average run length: " + (double) (runLengthTotal-1) /
	    (double) (runLengthEntries-1));
	for (int i = 0; i < histogramArraySize; i++)
	    {
	    System.out.println(i + ":" + histogramArray[i]);
	    }
    	}

    public static void addToHistogram(int runLength)
    	{
	if (runLength > (histogramArraySize-1))
	    histogramArray[histogramArraySize-1]++;
	else
	    histogramArray[runLength]++;
    	}
    }
    